/* Shoot or Die! - OpenGLDriver.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <iostream>
#include <SDL/SDL_opengl.h>
#include "OpenGLDriver.h"

OpenGLDriver::OpenGLDriver(int width, int height, int depth, std::string title, bool is_full)
		: WIDTH(width), HEIGHT(height), DEPTH(depth), TITLE(title), fullscreen(is_full) { 

	// set video mode
	SetVideoMode(WIDTH, HEIGHT, DEPTH);

	// initialize OpenGL
	ResizeSurface(WIDTH, HEIGHT);
	InitOpenGL();

	// set window title
	SDL_WM_SetCaption(TITLE.c_str(), TITLE.c_str());

	// check fullscreen
	/*if (fullscreen) {							// NOTE: toggle fullscreent doesn't work with windows
		fullscreen = !fullscreen;
		Fullscreen();
	}*/
}

OpenGLDriver::~OpenGLDriver() { }

void OpenGLDriver::SetVideoMode(int width, int height, int depth) {

	// set video mode
	//SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);				// NOTE: should acutally be enable but creates a segmentation fault
	screen = SDL_SetVideoMode(width, height, depth, SDL_OPENGL | SDL_FULLSCREEN);
	if (!screen) {
		std::cerr << "Unable to set video mode "
		     << width << "x" << height << "x" << depth << ": "
		     << SDL_GetError() << std::endl;
		exit(1);
	}
}

void OpenGLDriver::ResizeSurface(int width, int height) {

	if (!height) height = 1;

	glViewport(0, 0, width, height);

	// Set projection
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, width, height, 0, 0, 1);

	// Initialize modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void OpenGLDriver::InitOpenGL() {

	// initialize OpenGL
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	if (glGetError() != GL_NO_ERROR) {
		std::cerr << "Couldn't setup OpenGL\n";
		exit(1);
	}
}

void OpenGLDriver::Fullscreen() {
	if (!SDL_WM_ToggleFullScreen(SDL_GetVideoSurface())) {
		std::cerr << "Failed to toggle windowed/fullscreen mode: "
		          << SDL_GetError() << std::endl;
		exit(1);
	}

	fullscreen = !fullscreen;
}

void OpenGLDriver::ShowMouse(bool mouse) { SDL_ShowCursor(mouse); }

void OpenGLDriver::BeginDraw() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

void OpenGLDriver::EndDraw() { SDL_GL_SwapBuffers(); }
