/* Shoot or Die! - OpenGLGraphics.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_opengl.h>
#include "OpenGLGraphics.h"
#include "OpenGLImage.h"

OpenGLGraphics::OpenGLGraphics() {}
OpenGLGraphics::~OpenGLGraphics() {}

void OpenGLGraphics::PutPixel(int x, int y, int color) {

	// convert hex to rgb color
	Uint32 r = color & 0xff0000;
	r >>= 16;
	Uint32 g = color & 0x00ff00;
	g >>=8;
	Uint32 b = color & 0x0000ff;

	// draw pixel
	PutPixel(x, y, r, g, b);
}

void OpenGLGraphics::PutPixel(int x, int y, short r, short g, short b) {
	glBegin(GL_POINTS);
		glColor3ub(r, g, b);
		glVertex2f(x, y);
	glEnd();
}

void OpenGLGraphics::Blit(Image* source, Image* destination, Oblong* clip, int x, int y) {

	OpenGLImage* image = dynamic_cast<OpenGLImage*>(source);

	GLuint texture_handle = image->Texture();

	glBindTexture(GL_TEXTURE_2D, texture_handle);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	double texel_x = (double)clip->x / image->TextureWidth();
	double texel_y = (double)clip->y / image->TextureHeight();
	double texel_width = (double)(clip->x + clip->width) / image->TextureWidth();
	double texel_height = (double)(clip->y + clip->height) / image->TextureHeight();

	glBegin(GL_QUADS);

		// top-left 
		glTexCoord2f(texel_x, texel_y);
		glVertex3f(x, y, 0.0f);

		// top-right
		glTexCoord2f(texel_width, texel_y);
		glVertex3f(x + clip->width, y, 0);

		// bottom-right
		glTexCoord2f(texel_width, texel_height);
		glVertex3f(x + clip->width, y + clip->height, 0);

		// bottom-right
		glTexCoord2f(texel_x, texel_height);
		glVertex3f(x, y + clip->height, 0);

	glEnd();

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

void OpenGLGraphics::DrawImage(Image* img, int x, int y) {

	OpenGLImage* image = dynamic_cast<OpenGLImage*>(img);

	GLuint texture_handle = image->Texture();

	glBindTexture(GL_TEXTURE_2D, texture_handle);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glBegin(GL_QUADS);

		// top-left 
		glTexCoord2i(0, 0);
		glVertex3f(x, y, 0.0f);

		// bottom-left
		glTexCoord2i(1, 0);
		glVertex3f(x + image->TextureWidth(), y, 0);

		// bottom-right
		glTexCoord2i(1, 1);
		glVertex3f(x + image->TextureWidth(), y + image->TextureHeight(), 0);

		// top-right
		glTexCoord2i(0, 1);
		glVertex3f(x, y + image->TextureHeight(), 0);

	glEnd();

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}
