/* Shoot or Die! - OpenGLImage.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <SDL/SDL_image.h>
#include "OpenGLImage.h"

OpenGLImage::OpenGLImage() { }
OpenGLImage::~OpenGLImage() { glDeleteTextures(1, &texture_handle); }

Image* OpenGLImage::LoadImage(std::string filename, int colorkey) {

	// load image via SDL
	SDL_Surface* image = SDLLoadImage(filename, colorkey);

	// get image size
	image_width = image->w;
	image_height = image->h;

	// determine color depth of the image
	color_depth = image->format->BytesPerPixel;
	if (color_depth == 4) {
		if (image->format->Rmask == 0x000000ff) texture_format = GL_RGBA;
		else texture_format = GL_BGRA;
	}
	else if (color_depth == 3) {
		if (image->format->Rmask == 0x000000ff) texture_format = GL_RGB;
		else texture_format = GL_BGR;
	}
	else std::cerr << "not truecolor\n";

	// determine texture size
	texture_width = 1;
	texture_height = 1;

	while (texture_width < image->w) texture_width *= 2;
	while (texture_height < image->h) texture_height *= 2;

	// create texture
	int alpha = (SDL_BYTEORDER == SDL_LIL_ENDIAN) ? 0xff000000 : 0x000000ff;

	texture = new unsigned int[texture_width * texture_height];
	for (int y = 0; y < texture_height; y++)
		for (int x = 0; x < texture_width; x++) {
			if (x < image->w && y < image->h) {
				int color = *((Uint32*)image->pixels + y * image->w + x);
				texture[x + y * texture_width] = (color == colorkey) ? 0x00000000 : alpha | color;
			}
			else texture[x + y * texture_width] = 0x00000000;
		}

	// free image memory
	SDL_FreeSurface(image);

	// initialize OpenGL texture
	glGenTextures(1, &texture_handle);
	glBindTexture(GL_TEXTURE_2D, texture_handle);

	// create OpenGL texture
	glTexImage2D(GL_TEXTURE_2D, 0, color_depth, texture_width, texture_height, 0, texture_format, GL_UNSIGNED_BYTE, texture);
	delete[] texture;

	// specify filter
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);		// GL_NEAREST == fast; GL_LINEAR == best
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	return this; // texture
}

SDL_Surface* OpenGLImage::SDLLoadImage(std::string filename, int colorkey) {


	// load image into temporary buffer
	SDL_Surface* tbuf = IMG_Load(filename.c_str());
	if (!tbuf) {
		std::cerr << "Couldn't load '" << filename << "': "
		          << SDL_GetError() << std::endl;
		exit(1);
	}

	// optimize image
	SDL_Surface* image = SDL_DisplayFormat(tbuf);
	SDL_FreeSurface(tbuf);

	return image;
}

int OpenGLImage::Width() { return image_width; }
int OpenGLImage::Height() { return image_height; }
int OpenGLImage::TextureWidth() { return texture_width; }
int OpenGLImage::TextureHeight() { return texture_height; }

GLuint OpenGLImage::Texture() { return texture_handle; }
