/* Shoot or Die! - SDLDriver.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <iostream>
#include "SDLDriver.h"

SDLDriver::SDLDriver(int width, int height, int depth, std::string title, bool is_full)
		: WIDTH(width), HEIGHT(height), DEPTH(depth), TITLE(title), fullscreen(is_full) {

	// set video mode
	SetVideoMode(WIDTH, HEIGHT, DEPTH);

	// set window title
	SDL_WM_SetCaption(TITLE.c_str(), TITLE.c_str());

	// check fullscreen				// NOTE: toggle fullscreen doesn't work with windows
	/*if (fullscreen) {
		fullscreen = !fullscreen;
		Fullscreen();
	}*/
}

SDLDriver::~SDLDriver() { }

void SDLDriver::SetVideoMode(int width, int height, int depth) {

	// set video mode
	screen = SDL_SetVideoMode(width, height, depth, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
	if (!screen) {
		std::cerr << "Unable to set video mode "
		     << width << "x" << height << "x" << depth << ": "
		     << SDL_GetError() << std::endl;
		exit(1);
	}
}

void SDLDriver::Fullscreen() {
	if (!SDL_WM_ToggleFullScreen(SDL_GetVideoSurface())) {
		std::cerr << "Failed to toggle windowed/fullscreen mode: "
		          << SDL_GetError() << std::endl;
		exit(1);
	}

	fullscreen = !fullscreen;
}
void SDLDriver::ShowMouse(bool mouse) { SDL_ShowCursor(mouse); }

void SDLDriver::BeginDraw() {
	if (SDL_MUSTLOCK(screen)) SDL_LockSurface(screen);
	SDL_FillRect(screen, NULL, 0x000000);
}

void SDLDriver::EndDraw() {
	if (SDL_MUSTLOCK(screen)) SDL_UnlockSurface(screen);
	SDL_Flip(screen);
}
