/* Shoot or Die! - SDLGraphics.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <SDL/SDL_image.h>
#include "SDLGraphics.h"
#include "SDLImage.h"

SDLGraphics::SDLGraphics(SDL_Surface* surface) : screen(surface) {}
SDLGraphics::~SDLGraphics() {}

void SDLGraphics::PutPixel(int x, int y, int color) {
	*((Uint32*)screen->pixels + y * screen->w + x) = color;
}

void SDLGraphics::PutPixel(int x, int y, short r, short g, short b) {
	Uint32 color = SDL_MapRGB(screen->format, r, g, b);

	switch (screen->format->BytesPerPixel) {
		case 1: // 8-bpp
			*((Uint8*)screen->pixels + y * screen->pitch + x) = color;
			break;
		case 2: // 15-bpp or 16-bpp
			*((Uint16*)screen->pixels + y * screen->pitch/2 + x) = color;
			break;
		case 3: // 24-bpp
			Uint8 *pixel;
			pixel = (Uint8*)screen->pixels + y * screen->pitch + x * 3;
			if (SDL_BYTEORDER == SDL_LIL_ENDIAN) {
				pixel[0] = color;
				pixel[1] = color >> 8;
				pixel[2] = color >> 16;
			} else {
				pixel[2] = color;
				pixel[1] = color >> 8;
				pixel[0] = color >> 16;
			}
			break;
		case 4: // 32-bpp
			*((Uint32*)screen->pixels + y * screen->pitch/4 + x) = color;
			break;
	}
}

int SDLGraphics::GetPixel(int x, int y) {

	return *((Uint32*)screen->pixels + y * screen->w + x);
}

void SDLGraphics::Blit(Image* src, Image* dest, Oblong* rect, int x, int y) {

	SDLImage* source = dynamic_cast<SDLImage*>(src);

	// set destination coordinates
	SDL_Rect clip;
	clip.x = rect ? rect->x : 0;
	clip.y = rect ? rect->y : 0;
	clip.w = rect ? rect->width : source->Width();
	clip.h = rect ? rect->height : source->Height();
	
	SDL_Rect offset = { x, y };

	// blit image
	SDL_BlitSurface(source->Surface(), &clip, screen, &offset);
}

void SDLGraphics::DrawImage(Image* img, int x, int y) {

	SDLImage* image = dynamic_cast<SDLImage*>(img);

	SDL_Rect offset = { x, y };
	SDL_BlitSurface(image->Surface(), NULL, screen, &offset);
}
