/* Shoot or Die! - Sprite.cpp
 * Copyright (c) 2008, 2009 by Michael Borgmann <mail@lechuck.de>
 * This code is released under the ISC license
 */

#include <SDL/SDL.h>
#include "Sprite.h"

Sprite::Sprite(std::string filename, int colorkey, int fps) : xpos(0), ypos(0), xvel(0), yvel(0), frame(0), animated(false) {}

void Sprite::SetPosition(int x, int y) {
	xpos = x;
	ypos = y;
}

int Sprite::XVelocity(int vel) {
	return xvel = vel;
}

int Sprite::YVelocity(int vel) {
	return yvel = vel;
}

void Sprite::Move() {
	xpos += xvel;
	ypos += yvel;
}

void Sprite::Draw(Image* dest, Oblong* clip, int x, int y) {

	// blit sprite
	if (x == -1) x = xpos;
	if (y == -1) y = ypos;
	graphics->Blit(spritesheet, dest, clip, x, y);

	// calculate next frame
	if (animated && framerate) {

		int rest = SDL_GetTicks() - timer;

		if (rest > 1000 / framerate) {

			timer = SDL_GetTicks();

			if ((clip->x + clip->width) < spritesheet->Width()) {
				clip->x += clip->width;
				frame++;
			} else {
				clip->x = 0;
				frame = 0;
			}
		}

	}
}

bool Sprite::Animate(bool anim) {
	timer = SDL_GetTicks();
	return animated = anim;
}
